﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Npgsql;

namespace MigCdm
{
    class AppProc
    {
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        private List<CdmPropDto> dtoList;

        private List<CdmPropDto> duplicateList;

        public void getCdmInfo()
        {
            logger.Debug("getCdmInfo() start");

            try
            {
                DbUtil util = new DbUtil();
                //DB接続
                NpgsqlConnection conn = util.open();

                CdmPropDao dao = new CdmPropDao();


                //全件照会
                logger.Debug("ALL query start");

                dtoList = dao.query_new(conn);

                logger.Debug("ALL query end 件数：" + dtoList.Count);

                //重複リスト取得＆重複フラグオン　
                logger.Debug("check duplicate start");

                duplicateList = new List<CdmPropDto>();

                NpgsqlTransaction transaction = null;
                try
                {
                    transaction = conn.BeginTransaction();

                    duplicateList = dao.query_duplicate_all(conn); //30秒

                    foreach (var dto in duplicateList)
                    {
                        //重複リスト取得＆重複フラグオン
                        dao.update_duplicate(conn, dto);

 //                           logger.Debug("[" + dto.Bulk_id + "," + dto.Fname + "]" + "は重複しています。");
                    }
//                    if (duplicateList.Count > 0)
//                    {
                        transaction.Commit();
//                    }

                    logger.Debug("check duplicate end");
                }
                catch (NpgsqlException ex)
                {
                    transaction.Rollback();
                    logger.Debug("データベースの更新に失敗しました。" + ex.Message);
                }

                //DB切断
                util.close(conn);

            }
            catch(NpgsqlException ex)
            {
                logger.Error("例外発生：" + ex.Message);
            }
            finally{
                logger.Debug("getCdmInfo() end");

            }
        }
        public void download()
        {
            logger.Debug("download() start");

            foreach (CdmPropDto dto in duplicateList)
            {
                logger.Debug("重複データ全部：" + dto.Bulk_id + "," + dto.Fname);
            }

            // .NET4.5～使用可
            Task t1 = Task.Run(() => downloadProc(1));
            // .NET4.5～使用可
            Task t2 = Task.Run(() => downloadProc(2));

            // 全てのタスクが終了するまで待機
            Task.WaitAll(t1, t2);

            logger.Debug("download() end");
        }

        /// <summary>
        /// ワーカスレッドで実行されるメソッドです。
        /// </summary>
        private void downloadProc(int num)
        {
            logger.Debug("downloadProc() start num = " + num);

            logger.Debug("downloadProc() ALL query end 件数：" + dtoList.Count);

            int num1cnt = dtoList.Count / 2;

            if (num == 1)
            {
                for (int i = 0; i < num1cnt; i++)
                {
                    CdmPropDto dto = new CdmPropDto();

                    dto = dtoList[i];
                    if (duplicateList.Contains(dto))
                    {
                        logger.Debug("num, i, 重複データ : " + num + "," + i + ","+ dto.Fname);
                    }
                    else
                    {
                        String srcpath = "..\\07_1.jpg";
                        String dstpath = "..\\swfile\\" + dto.Fname + dto.Version+ ".jpg";

                        File.Copy(srcpath, dstpath);

                        //String fpath = "..\\swfile\\" + dto.Fname + ".txt";
                        //using (StreamWriter sw = new StreamWriter(fpath))
                        //{
                        //    sw.WriteLine(num + "," + dto.Fname);
                        //    File.Copy(@"E:\samurai1.txt", fpath);
                        //}
                    }
                    if (i == (num1cnt - 1)){
                        logger.Debug("Fname : " + dto.Fname);
                    }
                }

            }
            else if (num == 2)
            {
                for (int i = num1cnt; i < dtoList.Count; i++)
                {
                    CdmPropDto dto = new CdmPropDto();

                    dto = dtoList[i];
                    if (duplicateList.Contains(dto))
                    {
                        logger.Debug("num, i, 重複データ : " + num + ","+ i + "," + dto.Fname);
                    }
                    else
                    {
                        String srcpath = "..\\07_3.jpg";
                        String dstpath = "..\\swfile\\" + dto.Fname + dto.Version + ".jpg";

                        File.Copy(srcpath, dstpath);
                        //String fpath = "..\\swfile\\" + dto.Fname + ".txt";
                        //using (StreamWriter sw = new StreamWriter(fpath))
                        //{
                        //    sw.WriteLine(num + "," + dto.Fname);
                        //}
                    }

                    if (i == (dtoList.Count - 1))
                    {
                        logger.Debug("Fname : " + dto.Fname);
                    }

                }

            }
            logger.Debug("downloadProc() end num = " + num);
        }

    }
}
