﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Npgsql;
using NpgsqlTypes;

namespace MigCdm
{
    class CdmPropDao
    {
        private static log4net.ILog logger = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        public List<CdmPropDto> query(NpgsqlConnection conn)
        {
            //            logger.Debug("query() start");

            // 発行するselect文
            //            string sql = @"select bulk_id, fname, create_time from t_cdmprop where id < 100000";
            string sql = @"select bulk_id, fname, create_time from t_cdmprop ";

            NpgsqlCommand cmd = new NpgsqlCommand(sql, conn);

            // クエリ実行
            List<CdmPropDto> sampledataList = new List<CdmPropDto>();

            using (NpgsqlDataReader dr = cmd.ExecuteReader())
            {
                // レコードなし
                //if (dr.HasRows == false)
                //{
                //    // この処理はなくてもOK
                //}

                while (dr.Read())
                {
                    // NULL の場合は、初期値を設定するようにする
                    // データベースの型に合わせて格納する変数の型も変更する
                    CdmPropDto dto = new CdmPropDto();

                    dto.Bulk_id = (String)dr["bulk_id"];
                    dto.Fname = (String)dr["fname"];

                    dto.Create_time = (String)dr["create_time"];

                    // 取得したデータを云々・・の処理など
                    String logmsg = dto.Bulk_id + "," + dto.Fname + "," + dto.Create_time;

                    //                    logger.Debug("query() 結果：" + logmsg);

                    sampledataList.Add(dto);
                }
                //                logger.Debug("query() end");

            }
            return sampledataList;
        }
        public List<CdmPropDto> query_new(NpgsqlConnection conn)
        {
            //            logger.Debug("query() start");

            // 発行するselect文
            //            string sql = @"select bulk_id, fname, create_time from t_cdmprop where id < 100000";
            //string sql = @"select fname from t_cdmprop ";

            // versionが最大のレコードのみ取得
            //String sql = "SELECT a.bulk_id, a.fname, a.version , a.create_time, a.owner, a.approval_user, a.approval_date FROM t_cdmprop AS a ";
            //sql += "INNER JOIN(SELECT fname, MAX(version) AS maxver ";
            //sql += "FROM t_cdmprop ";
            //sql += "GROUP BY fname) AS b ";
            //sql += "ON a.fname = b.fname AND a.version = b.maxver";

            // 全version取得
            String sql = "SELECT a.bulk_id, a.fname, a.version , a.create_time, a.owner, a.approval_user, a.approval_date FROM t_cdmprop AS a ";
            sql += " ORDER BY a.fname, a.version ";

            NpgsqlCommand cmd = new NpgsqlCommand(sql, conn);

            //コマンド　タイムアウト設定
            cmd.CommandTimeout = 60;    

            // クエリ実行
            List<CdmPropDto> sampledataList = new List<CdmPropDto>();

            using (NpgsqlDataReader dr = cmd.ExecuteReader())
            {
                // レコードなし
                //if (dr.HasRows == false)
                //{
                //    // この処理はなくてもOK
                //}

                while (dr.Read())
                {
                    // NULL の場合は、初期値を設定するようにする
                    // データベースの型に合わせて格納する変数の型も変更する
                    CdmPropDto dto = new CdmPropDto();

                    dto.Bulk_id = (String)dr["bulk_id"];
                    dto.Fname = (String)dr["fname"];
                    dto.Version = (int)dr["version"];

                    dto.Create_time = (String)dr["create_time"];
                    dto.Approval_date = (DateTime)dr["Approval_date"];

                    // 取得したデータを云々・・の処理など
                    String logmsg = dto.Bulk_id + "," + dto.Fname + "," + dto.Create_time;

                    //                    logger.Debug("query() 結果：" + logmsg);

                    sampledataList.Add(dto);
                }
                //                logger.Debug("query() end");

            }
            return sampledataList;
        }
        public List<CdmPropDto> query_duplicate(NpgsqlConnection conn, CdmPropDto indto)
        {
            logger.Debug("query() start");

            // 発行するselect文
            string sql = @"select bulk_id, fname from t_cdmprop where fname = :fname and doctype <> 3";

            NpgsqlCommand cmd = new NpgsqlCommand(sql, conn);

            cmd.Parameters.Add(new NpgsqlParameter("fname", NpgsqlDbType.Varchar)); //パラメーターの追加
            cmd.Parameters["fname"].Value = indto.Fname; //パラメーターに値をセット

            // クエリ実行
            List<CdmPropDto> sampledataList = new List<CdmPropDto>();

            using (NpgsqlDataReader dr = cmd.ExecuteReader())
            {
                // レコードなし
                if (dr.HasRows == false)
                {
                    // この処理はなくてもOK
                }

                while (dr.Read())
                {
                    // NULL の場合は、初期値を設定するようにする
                    // データベースの型に合わせて格納する変数の型も変更する
                    CdmPropDto dto = new CdmPropDto();

                    dto.Bulk_id = (String)dr["bulk_id"];
                    dto.Fname = (String)dr["fname"];

//                    dto.Create_time = (String)dr["create_time"];

                    // 取得したデータを云々・・の処理など
                    String logmsg = dto.Bulk_id + "," + dto.Fname + "," + dto.Create_time;

                    logger.Debug("query() 結果：" + logmsg);

                    sampledataList.Add(dto);
                }
                logger.Debug("query() end");

            }
            return sampledataList;
        }
        public List<CdmPropDto> query_duplicate_all(NpgsqlConnection conn)
        {
            logger.Debug("query() start");

            // 発行するselect文
            String sql = "SELECT a.bulk_id, a.fname, a.version, a.create_time, a.owner, a.approval_user, a.approval_date FROM t_cdmprop a ";
            sql += "INNER JOIN( ";
            sql += "    SELECT fname, version FROM t_cdmprop ";
            sql += "    GROUP BY fname, version ";
            sql += "    HAVING COUNT(fname) > 1 ";
            sql += ")  b ";
            sql += "ON a.fname = b.fname AND a.version = b.version ";
            sql += "ORDER BY a.fname";



            NpgsqlCommand cmd = new NpgsqlCommand(sql, conn);

            //コマンド　タイムアウト設定
            cmd.CommandTimeout = 50;

            // クエリ実行
            List<CdmPropDto> cdmPropDataList = new List<CdmPropDto>();

            using (NpgsqlDataReader dr = cmd.ExecuteReader())
            {
                // レコードなし
                //if (dr.HasRows == false)
                //{
                //    // この処理はなくてもOK
                //}

                while (dr.Read())
                {
                    // NULL の場合は、初期値を設定するようにする
                    // データベースの型に合わせて格納する変数の型も変更する
                    CdmPropDto dto = new CdmPropDto();

                    dto.Bulk_id = (String)dr["bulk_id"];
                    dto.Fname = (String)dr["fname"];
                    dto.Version = (int)dr["version"];

                    dto.Create_time = (String)dr["create_time"];
                    dto.Approval_user = (String)dr["Approval_user"];
                    dto.Approval_date = (DateTime)dr["Approval_date"];

                    // 取得したデータを云々・・の処理など
                    String logmsg = dto.Bulk_id + "," + dto.Fname + "," + dto.Create_time;

                    //                    logger.Debug("query() 結果：" + logmsg);

                    cdmPropDataList.Add(dto);
                }
            }
            return cdmPropDataList;

        }
        public int update_duplicate(NpgsqlConnection conn, CdmPropDto indto)
        {
            logger.Debug("query() start");

            // 発行するselect文
            string sql = @"update t_cdmprop set duplicate = 1 where fname = :fname ";

            int rslt = 0;
            try
            {
                NpgsqlCommand cmd = new NpgsqlCommand(sql, conn);

                cmd.Parameters.Add(new NpgsqlParameter("fname", NpgsqlDbType.Varchar)); //パラメーターの追加
                cmd.Parameters["fname"].Value = indto.Fname; //パラメーターに値をセット

                rslt = cmd.ExecuteNonQuery();

            }
            catch (NpgsqlException ex)
            {
                throw;
            }
            return rslt;
        }

    }
}
